'use strict';

// src/constant.ts
var SLOTS_NAMES = [
  "default",
  "title",
  "footer"
];
var VISIBLE_KEY = "update:visible";

// src/utils.ts
function isComponent(content) {
  return !Object.keys(content).some((key) => SLOTS_NAMES.includes(key));
}
function resolveSlots(instance, vm) {
  let { content } = instance;
  const scopedSlots = {};
  const children = [];
  if (content) {
    const resolve = instance.resolve;
    const h = vm.$createElement;
    const on = {
      close(...args) {
        vm.$data.visible = false;
        let value;
        if (args.length === 0)
          value = void 0;
        else if (args.length === 1)
          value = args[0];
        else
          value = args;
        resolve(value);
      }
    };
    if (isComponent(content)) {
      content = {
        default: {
          component: content,
          propsData: {}
        }
      };
    }
    Object.entries(content).forEach(([key, value]) => {
      if (!value.component) {
        value = {
          component: value
        };
      }
      const { propsData = {}, component } = value;
      if (component) {
        const vnode = h(component, { props: propsData, on });
        scopedSlots[key] = () => [vnode];
        if (key === "footer") {
          children.push(h("template", { slot: key }, [vnode]));
        }
      }
    });
  }
  return { scopedSlots, children };
}
function resolveOptions(instance, vm) {
  const { options: { open, opened, close, closed, ...props } } = instance;
  const on = {};
  const events = { open, opened, close, closed };
  Object.entries(events).filter(([, value]) => value).forEach(([key, value]) => on[key] = value);
  return {
    props: {
      ...props,
      visible: vm.$data.visible
    },
    on: {
      ...on,
      [VISIBLE_KEY](value) {
        vm.$data.visible = value;
      }
    }
  };
}

// src/index.ts
function Dialog(Vue) {
  return class Dialog {
    constructor(options = {}) {
      this.globalOptions = options;
      this.options = {};
      this.vm = null;
    }
    dialog(content, options = {}) {
      if (!content || typeof content !== "object" || !Object.keys(content).length) {
        return Promise.reject(new Error("The content shown in 'el-dialog' component cannot be empty"));
      }
      this.options = Object.assign({}, this.globalOptions, options);
      this.content = content;
      if (!this.vm) {
        this.vm = this.getInstance();
      }
      const vm = this.vm;
      document.body.appendChild(vm.$el);
      vm.$data.visible = this.options.visible;
      return new Promise((resolve, reject) => {
        this.resolve = resolve;
        this.reject = reject;
      });
    }
    getInstance() {
      const component = this.createComponent();
      const DialogCtor = Vue.extend(component);
      return new DialogCtor().$mount();
    }
    createComponent() {
      const instance = this;
      return {
        data() {
          return {
            visible: false
          };
        },
        render(h) {
          const { props, on } = resolveOptions(instance, this);
          const { scopedSlots, children } = resolveSlots(instance, this);
          return h(
            "ElDialog",
            {
              props,
              on,
              scopedSlots
            },
            children
          );
        },
        // only used to test
        destroyed() {
          console.log("destroyed");
        }
      };
    }
    destroy() {
      const { vm } = this;
      if (!vm)
        return true;
      if (vm.$el && vm.$el.parentNode) {
        vm.$el.parentNode.removeChild(vm.$el);
      }
      vm.$destroy();
      this.vm = null;
      return true;
    }
  };
}
Dialog.install = (Vue, options = {}) => {
  const DialogClass = Dialog(Vue);
  const dialog = new DialogClass(options);
  Vue.prototype.$dialog = dialog.dialog.bind(dialog);
};

module.exports = Dialog;
